from polytope_feature import shapes
from shapely.geometry import Polygon

from ..feature import Feature


def get_area(points):
    x = []
    y = []
    for point in points:
        x.append(point[0])
        y.append(point[1])
    pgon = Polygon(zip(x, y))
    return pgon.area


class Polygons(Feature):
    def __init__(self, feature_config, client_config):
        assert feature_config.pop("type") == "polygon"
        self.shape = feature_config.pop("shape")
        if type(self.shape[0][0]) is not list:
            if len(self.shape) > client_config.polygonrules.max_points:
                raise ValueError(
                    f"Number of points {len(self.shape)} exceeds the maximum of {client_config.polygonrules.max_points}"  # noqa: E501
                )
            if get_area(self.shape) > client_config.polygonrules.max_area:
                raise ValueError(
                    f"Area of polygon {get_area(self.shape)} exceeds the maximum of size of {client_config.polygonrules.max_area} degrees"  # noqa: E501
                )
            self.shape = [self.shape]
        else:
            len_polygons = 0
            area_polygons = 0
            for polygon in self.shape:
                len_polygons += len(polygon)
                area_polygons += get_area(polygon)
            if len_polygons > client_config.polygonrules.max_points:
                raise ValueError(
                    f"Number of points {len_polygons} exceeds the maximum of {client_config.polygonrules.max_points}"  # noqa: E501
                )
            if area_polygons > client_config.polygonrules.max_area:
                raise ValueError(
                    f"Area of polygon {area_polygons} exceeds the maximum of size of {client_config.polygonrules.max_area} degrees"  # noqa: E501
                )

        assert (
            len(feature_config) == 0
        ), f"Unexpected keys in config: {feature_config.keys()}"

    def get_shapes(self):
        # coordinates = get_coords(self.df)
        polygons = []
        for polygon in self.shape:
            points = []
            for point in polygon:
                points.append([point[0], point[1]])
            polygons.append(shapes.Polygon(["latitude", "longitude"], points))
        return [shapes.Union(["latitude", "longitude"], *polygons)]

    def incompatible_keys(self):
        return []

    def coverage_type(self):
        return "MultiPoint"

    def name(self):
        return "Polygon"

    def parse(self, request, feature_config):
        if feature_config["type"] != "polygon":
            raise ValueError("Feature type must be polygon")
        if "step" in request and "number" in request:
            step = request["step"].split("/")
            number = request["number"].split("/")
            if len(step) > 1 and len(number) > 1:
                raise ValueError(
                    "Multiple steps and numbers not yet supported for polygon feature, this will be supported in the future"  # noqa: E501
                )
        return request



from polytope_feature import shapes

from ..feature import Feature


class BoundingBox(Feature):
    def __init__(self, feature_config, client_config):
        assert feature_config.pop("type") == "boundingbox"
        self.points = feature_config.pop("points", [])
        self.axes = feature_config.pop("axes", [])

        if "axes" in feature_config:
            raise ValueError(
                "Bounding box does not have axes in feature, did you mean axes?"  # noqa: E501
            )

        assert (
            len(feature_config) == 0
        ), f"Unexpected keys in config: {feature_config.keys()}"

        area_bb = abs(self.points[0][0] - self.points[1][0]) * abs(
            self.points[0][1] - self.points[1][1]
        )  # noqa: E501
        print("area of bounding box:", area_bb)
        if area_bb > client_config.polygonrules.max_area:
            raise ValueError(
                f"Area of Bounding Box {area_bb} exceeds the maximum size of {client_config.polygonrules.max_area} degrees"  # noqa: E501
            )

    def get_shapes(self):
        # Time-series is a squashed box from start_step to start_end for each point  # noqa: E501
        if len(self.points[0]) == 2:
            return [
                shapes.Union(
                    ["latitude", "longitude"],
                    *[
                        shapes.Box(
                            ["latitude", "longitude"],
                            lower_corner=[
                                self.points[0][0],
                                self.points[0][1],
                            ],  # noqa: E501
                            upper_corner=[
                                self.points[1][0],
                                self.points[1][1],
                            ],  # noqa: E501
                        )
                    ],
                )
            ]
        else:
            return [
                shapes.Union(
                    ["latitude", "longitude", "levelist"],
                    *[
                        shapes.Box(
                            ["latitude", "longitude", "levelist"],
                            lower_corner=[
                                self.points[0][0],
                                self.points[0][1],
                                self.points[0][2],
                            ],
                            upper_corner=[
                                self.points[1][0],
                                self.points[1][1],
                                self.points[1][2],
                            ],
                        )
                    ],
                )
            ]

    def incompatible_keys(self):
        return []

    def coverage_type(self):
        return "MultiPoint"

    def name(self):
        return "Bounding Box"

    def parse(self, request, feature_config):
        if feature_config["type"] != "boundingbox":
            raise ValueError("Feature type must be boudningbox")
        if "step" in request and "number" in request:
            step = request["step"].split("/")
            number = request["number"].split("/")
            if len(step) > 1 and len(number) > 1:
                raise ValueError(
                    "Multiple steps and numbers not yet supported for Bounding Box feature, this will be added in the future"  # noqa: E501
                )
        if len(feature_config["points"]) != 2:
            raise ValueError(
                "Bounding box must have only two points in points"
            )  # noqa: E501
        if "axis" in feature_config:
            raise ValueError(
                "Bounding box does not have axis in feature, did you mean axes?"  # noqa: E501
            )
        if "axes" not in feature_config:
            for point in feature_config["points"]:
                if len(point) != 2:
                    raise ValueError(
                        "For Bounding Box each point must have only two values unless axes is specified"  # noqa: E501
                    )
        else:
            for point in feature_config["points"]:
                if len(point) != len(feature_config["axes"]):
                    raise ValueError(
                        "Bounding Box points must have the same number of values as axes"  # noqa: E501
                    )
            if "axes" in feature_config:
                if ("levelist" in feature_config["axes"]) and (
                    "levelist" in request
                ):  # noqa: E501
                    raise ValueError(
                        "Bounding Box axes is overspecified in request"
                    )  # noqa: E501

        return request



from polytope_feature import shapes

from ..feature import Feature


class Path(Feature):
    def __init__(self, feature_config, client_config):
        assert feature_config.pop("type") == "trajectory"
        self.points = feature_config.pop("points", [])
        if "radius" in feature_config:
            self.radius = feature_config.pop("radius")
        if "axes" in feature_config:
            self.axes = feature_config.pop("axes")
        if "axis" in feature_config:
            raise ValueError(
                "Trajectory does not have axis in feature, did you mean axes?"  # noqa: E501
            )

        assert (
            len(feature_config) == 0
        ), f"Unexpected keys in config: {feature_config.keys()}"

    def get_shapes(self):
        # Time-series is a squashed box from start_step to start_end for each point  # noqa: E501
        if len(self.points[0]) == 2:
            return [
                shapes.Path(
                    ["latitude", "longitude"],
                    shapes.Box(
                        ["latitude", "longitude"],
                        [0, 0],
                        [self.radius, self.radius],
                    ),
                    *self.points,
                )
            ]
        elif len(self.points[0]) == 3:
            if self.axes[2] == "step":
                return [
                    shapes.Path(
                        ["latitude", "longitude", "step"],
                        shapes.Box(
                            ["latitude", "longitude", "step"],
                            [0, 0, 0],
                            [self.radius, self.radius, 0],
                        ),
                        *self.points,
                    )
                ]
            else:
                return [
                    shapes.Path(
                        ["latitude", "longitude", "levelist"],
                        shapes.Box(
                            ["latitude", "longitude", "levelist"],
                            [0, 0, 0],
                            [self.radius, self.radius, self.radius],
                        ),
                        *self.points,
                    )
                ]
        elif len(self.points[0]) == 4:
            return [
                shapes.Path(
                    ["latitude", "longitude", "levelist", "step"],
                    shapes.Box(
                        ["latitude", "longitude", "levelist", "step"],
                        [0, 0, 0, 0],
                        [self.radius, self.radius, self.radius, 0],
                    ),
                    *self.points,
                )
            ]

    def incompatible_keys(self):
        return []

    def coverage_type(self):
        return "Trajectory"

    def name(self):
        return "Path"

    def parse(self, request, feature_config):
        if feature_config["type"] != "trajectory":
            raise ValueError("Feature type must be trajectory")
        if "radius" not in feature_config:
            raise ValueError("Radius must be specified in request")
        if "step" in request and "number" in request:
            step = request["step"].split("/")
            number = request["number"].split("/")
            if len(step) > 1 and len(number) > 1:
                raise ValueError(
                    "Multiple steps and numbers not yet supported for trajectory feature"  # noqa: E501
                )
        if "step" in request:
            step = request["step"].split("/")
            if len(step) > 1:
                raise ValueError(
                    "Multiple steps not yet supported for trajectory feature"
                )
        if "axes" not in feature_config:
            for point in feature_config["points"]:
                if len(point) != 4:
                    raise ValueError(
                        "For Trajectory each point must have only two values unless axes is specified, point must have form ['latitude', 'longitude', 'levelist', 'step']"  # noqa: E501
                    )
        else:
            for point in feature_config["points"]:
                if len(point) != len(feature_config["axes"]):
                    raise ValueError(
                        "Trajectory points must have the same number of values as axes"  # noqa: E501
                    )
            if ("levelist" in feature_config["axes"]) and (
                "levelist" in request
            ):  # noqa: E501
                raise ValueError(
                    "Trajectory level axes is overspecified in request"
                )  # noqa: E501
            if feature_config["axes"] == [
                "latitude",
                "longitude",
                "levelist",
            ]:  # noqa: E501
                raise ValueError(
                    "['latitude', 'longitude', 'levelist'] not yet implemented"
                )
        if len(feature_config["points"]) < 2:
            raise ValueError(
                "Trajectory must have atleast two values in points"
            )  # noqa: E501

        return request
